// HParaView.cpp : implementation of the HParaView class
// 

#include "HParaView.h"
#include "HParaModel.h"
#include "HPSelectionSet.h"
#include "utility.h"
#include <assert.h>
#include "hc.h"
#include "hp_bridge.h"

#define MAX_BOOLEAN_TOOLS 1024


HParaView::HParaView(HBaseModel *model,
    char * 		alias,	
    char *		driver_type,
	char *		instance_name,
    long		window_handle,
    long		colormap) : HBaseView(model, alias, driver_type, instance_name, window_handle, colormap)

{
}

// app-specific init function
void HParaView::Init()
{
	HParaModel * model = (HParaModel *)GetModel();

	// call base's init function first to get the default HOOPS hierarchy for the view
	HBaseView::Init();

	// create our app-specific Selection object and initialize
	m_pSelection = new HPSelectionSet();
    m_pSelection->Init();

	// setup a global HOOPS colormap
	//HC_QSet_Color_Map("?driver", "black, white, red, green, blue, orange, yellow, magenta");
	HC_Define_System_Options("no warnings");

    // app-specific scene Defaults

    HC_Open_Segment_By_Key(m_ViewKey);
		HC_Set_Color_By_Index("windows", 0);
		HC_Set_Selectability("everything = off");
	HC_Close_Segment();

	// set up some scene defaults
	HC_Open_Segment_By_Key(m_SceneKey);

	  HC_Set_Color_By_Index("faces", 2);
	  HC_Set_Color_By_Index("text, lights", 1);
	  HC_Set_Color_By_Index("edges, lines", 1);
	  HC_Set_Color_By_Index("markers", 1);
	  HC_Set_Rendering_Options("no color interpolation, no color index interpolation");
	  HC_Set_Visibility ("lights = (faces = on, edges = off), markers = off, faces=on, edges=off, lines=off, text = on");
	  HC_Set_Selectability("everything = off, faces = on");

	  // new segment for temporary construction geometry
	  m_construction_segment_key = HC_KOpen_Segment ("construction");
		HC_Set_Heuristics("quick moves");
		HC_Set_Visibility("faces = off, edges = on, lines = on");
	  HC_Close_Segment ();

	  m_parts_seg = HC_KCreate_Segment("parts");
	  
	HC_Close_Segment();

    // windowspace (overlay) defaults
    HC_Open_Segment_By_Key(m_WindowspaceKey);
	  HC_Set_Color_By_Index ("geometry", 3);
	  HC_Set_Color_By_Index ("window contrast", 1);
	  HC_Set_Color_By_Index ("windows", 1);

	  HC_Set_Visibility("markers=on");
	  HC_Set_Color("markers = green, lines = green");
	  HC_Set_Marker_Symbol("+");
	  HC_Set_Selectability("off");
    HC_Close_Segment();	

}


// perform a boolean operation on selected items
void HParaView::BooleanOperation(PK_boolean_function_t	operation)
{
	int			i, j, count = 0;
	PK_BODY_t	target = 0;
	PK_BODY_t	temp_body = 0;
	int			number_of_tools = 0;
	PK_BODY_t	tools[MAX_BOOLEAN_TOOLS];
	HC_KEY		key, *keys;

    keys = (HC_KEY*)malloc(10*sizeof(HC_KEY));

 
	int numSelections;
	HPSelectionSet* selection = NULL;

	// get ptr to selection object
	selection = (HPSelectionSet *)GetSelection();
	assert(selection);

	// get the number of selected items
	numSelections = selection->GetSize();

	// must have at least 2 selected items for a boolean operation
	if (numSelections < 2)
		return;

	// First, determine the tagid of the target body based on the first elt in the selection list
	target = HP_Compute_TagID(selection->GetAt(0), PK_CLASS_body);
	if (target < 1)
	{
//		AfxMessageBox("The first selected object is not a valid Parasolid entity.");
		return;
	}


	// Then, determine the tools which will operate on the body
	for (i = 1; i < numSelections && number_of_tools < MAX_BOOLEAN_TOOLS; i++)
	{
		temp_body = HP_Compute_TagID(selection->GetAt(i), PK_CLASS_body);
		if (temp_body < 1)
		{
			return;
		}
		
		// check to see if the parasolid entity "temp_body" is the same as 
		// an entity we've already put into 
		for (j = 0; j < number_of_tools; j++)
		{
			// break on dupicate body, we already have this body in the tools array
			if (temp_body == tools[j])
				break;
		}
		// need to also compare target against this body, we don't want
		// to perform boolean operations between an entity and itself...
		// j == number_of_tools if and only if no match was found above.
		if ((j == number_of_tools) && (temp_body != target))
		{
			// no duplicate body found, add this body to the tools array

			tools[number_of_tools] = temp_body;
			number_of_tools++;
		}
	}

	// if there are no valid tools to operation on the target, return
    if (number_of_tools < 1) return;

	// deselect everything that's been selected BEFORE deleting geometry.
	selection->DeSelectAll();

	// now blow away ALL of the geometry associated with target
	count = HP_Compute_Geometry_Keys(target, &key, "bodies", 1);
	assert (count == 1);
	HC_Delete_By_Key(key);


	// now blow away ALL of the geometry associated with tools.
	for (j = 0; j < number_of_tools; j++)
	{
		count = HP_Compute_Geometry_Keys(tools[j], &key, "bodies", 1);
		assert (count == 1);
		HC_Delete_By_Key(key);
	}


    // perform the boolean operation and render the resulting parasolid entity
	HC_Open_Segment_By_Key(((HParaModel*)GetModel())->m_parts_seg);
		parasolid_boolean(operation, target, number_of_tools, tools);
	HC_Close_Segment();

    Update();	
}